/*
 * struct.h for the server of an xtrek socket protocol.
 */
#include "copyright.h"

#define REQFREE 0		/* To say that there is no request pending */
#define REQWHO 1		/* Who is below this number? */
#define REQDEAD 2		/* I am going bye bye.. */

struct status {
   int             active;
   unsigned int    wait, count;
   unsigned int    number, request, answer;
   unsigned char   tourn;	/* Tournament mode? */
   /* These stats only updated during tournament mode */
   unsigned int    armsbomb, planets, kills, losses, time;
   double	   timeprod;
   int             gameup;
#ifdef BASEPRACTICE
   double	   dami, damr;	/* damage inflicted, damage repaired */
#endif
};

#define PFREE 0
#define POUTFIT 1
#define PALIVE 2
#define PEXPLODE 3
#define PDEAD 4

#define PFSHIELD	   0x0001
#define PFREPAIR	   0x0002
#define PFBOMB		   0x0004
#define PFORBIT		   0x0008
#define PFCLOAK		   0x0010
#define PFWEP		   0x0020
#define PFENG		   0x0040
#define PFROBOT		   0x0080
#define PFBEAMUP	   0x0100
#define PFBEAMDOWN	   0x0200
#define PFSELFDEST	   0x0400
#define PFGREEN		   0x0800
#define PFYELLOW	   0x1000
#define PFRED		   0x2000
#define PFPLOCK		   0x4000	/* Locked on a player */
#define PFPLLOCK	   0x8000	/* Locked on a planet */
#define PFCOPILOT	  0x10000	/* Allow copilots */
#define PFWAR		  0x20000	/* computer reprogramming for war */
#define PFPRACTR	  0x40000	/* practice type robot (no kills) */
#define PFDOCK            0x80000	/* true if docked to a starbase */
#define PFREFIT          0x100000	/* true if about to refit */
#define PFREFITTING	 0x200000	/* true if currently refitting */
#define PFTRACT  	 0x400000	/* tractor beam activated */
#define PFPRESS  	 0x800000	/* pressor beam activated */
#define PFDOCKOK	0x1000000	/* docking permission */
#define PFSEEN		0x2000000	/* seen by enemy on galactic map? */

#ifdef SNAKEPATROL
#define PFSNAKE		0x4000000	/* the snake charmer */
#endif

#ifdef BIRD
#define PFBIRD	       0x08000000	/* bird man */
#endif

#ifdef ORGANIAN
#define PFORGANIAN     0x10000000
#endif

#ifdef CYBORGS
#define PFCYBORG       0x20000000	/* tag cyborgs */
#endif

#define PFTWARP	       0x40000000	/* isae -- SB transwarp */

#ifdef BASEPRACTICE
#define PFBPROBOT      0x80000000
#endif

#define KQUIT		0x01	/* Player quit */
#define KTORP		0x02	/* killed by torp */
#define KPHASER		0x03	/* killed by phaser */
#define KPLANET		0x04	/* killed by planet */
#define KSHIP		0x05	/* killed by other ship */
#define KDAEMON		0x06	/* killed by dying daemon */
#define KWINNER		0x07	/* killed by a winner */
#define KGHOST		0x08	/* killed because a ghost */
#define KGENOCIDE	0x09	/* killed by genocide */
#define KPROVIDENCE	0x0a	/* killed by a hacker */
#define KPLASMA         0x0b	/* killed by a plasma torpedo */
/* isae - My attempt to handle chain reaction */
#define KTORP2          0x0c            /* killed by detted torps */
#define KSHIP2          0x0d            /* chain-reaction explosions */

struct team {
   int             s_turns;	/* turns till another starbase is legal */
   int             s_surrender;	/* minutes until this team surrenders */
   int             s_plcount;	/* how many planets this team owns */
};

#define SURRSTART 1
#define SURREND 4
#define SURRLENGTH 30		/* was 45 */

#define NUM_TYPES 9
#define SCOUT 0
#define DESTROYER 1
#define CRUISER 2
#define BATTLESHIP 3
#define ASSAULT 4
#define STARBASE 5
#define GALAXY 6
#define ATT 7

struct ship {
   int             s_turns;
   short           s_accs;
   short           s_torpdamage;
   short           s_phaserdamage;
   short           s_phaserfuse;
   short           s_plasmadamage;
   short           s_torpspeed;
   short           s_torpfuse;
   short           s_torpturns;
   short           s_plasmaspeed;
   short           s_plasmafuse;
   short           s_plasmaturns;
   int             s_maxspeed;
   short           s_repair;
   int             s_maxfuel;
   short           s_torpcost;	/* has to do with wtemp needed to fire torps */
   short           s_plasmacost;
   short           s_phasercost;
   short           s_detcost;
   short           s_warpcost;
   short           s_cloakcost;
   short           s_recharge;
   int             s_accint;
   int             s_decint;
   int             s_maxshield;
   int             s_maxdamage;
   int             s_maxegntemp;
   int             s_maxwpntemp;
   short           s_egncoolrate;
   short           s_wpncoolrate;
   short           s_maxarmies;
   short           s_width;
   short           s_height;
   short           s_type;
   short           s_mass;	/* to guage affect of tractor beams */
   short           s_tractstr;	/* Strength of tractor beam */
   float           s_tractrng;
};

struct stats {
   double          st_maxkills;	/* max kills ever */
   int             st_kills;	/* how many kills */
   int             st_losses;	/* times killed */
   int             st_armsbomb;	/* armies bombed */
   int             st_planets;	/* planets conquered */
   int             st_ticks;	/* Ticks I've been in game */
   int             st_tkills;	/* Kills in tournament play */
   int             st_tlosses;	/* Losses in tournament play */
   int             st_tarmsbomb;/* Tournament armies bombed */
   int             st_tplanets;	/* Tournament planets conquered */
   int             st_tticks;	/* Tournament ticks */
   /* SB stats are entirely separate */
   int             st_sbkills;	/* Kills as starbase */
   int             st_sblosses;	/* Losses as starbase */
   int             st_sbticks;	/* Time as starbase */
   double          st_sbmaxkills;	/* Max kills as starbase */
   long            st_lastlogin;/* Last time this player was played */
   int             st_flags;	/* Misc option flags */
   char            st_keymap[96];	/* keymap for this player */
   int             st_rank;	/* Ranking of the player */
#ifdef BASEPRACTICE
   double	   st_dami;	/* damage inflicted */
   double	   st_damr;	/* damage repaired */
#endif
};

#define ST_MAPMODE      1
#define ST_NAMEMODE     2
#define ST_SHOWSHIELDS  4
#define ST_KEEPPEACE    8
#define ST_SHOWLOCAL    16	/* two bits for these two */
#define ST_SHOWGLOBAL   64

struct player {
   int             p_no;
   int             p_updates;	/* Number of updates ship has survived */
   int             p_status;	/* Player status */
   unsigned int    p_flags;	/* Player flags */
   char            p_name[16];
   char            p_login[16];
#ifdef LONGHOST
   char            p_monitor[64];	/* Monitor being played on */
#else
   char            p_monitor[16];	/* Monitor being played on */
#endif
   char            p_mapchars[2];	/* Cache for map window image */
   struct ship     p_ship;	/* Personal ship statistics */
   int             p_x;
   int             p_y;
   unsigned char   p_dir;	/* Real direction */
   unsigned char   p_desdir;	/* desired direction */
   int             p_subdir;	/* fraction direction change */
   int             p_speed;	/* Real speed */
   short           p_desspeed;	/* Desired speed */
   int             p_subspeed;	/* Fractional speed */
   short           p_team;	/* Team I'm on */
   int             p_damage;	/* Current damage */
   int             p_subdamage;	/* Fractional damage repair */
   int             p_shield;	/* Current shield power */
   int             p_subshield;	/* Fractional shield recharge */
   short           p_cloakphase;/* Drawing stage of cloaking
				 * engage/disengage. */
   short           p_ntorp;	/* Number of torps flying */
   short           p_nplasmatorp;	/* Number of plasma torps active */
   char            p_hostile;	/* Who my torps will hurt */
   char            p_swar;	/* Who am I at sticky war with */
   float           p_kills;	/* Enemies killed */
   short           p_planet;	/* Planet orbiting or locked onto */
   short           p_playerl;	/* Player locked onto */
   short           p_armies;
   int             p_fuel;
   short           p_explode;	/* Keeps track of final explosion */
   int             p_etemp;
   short           p_etime;
   int             p_wtemp;
   short           p_wtime;
   short           p_whydead;	/* Tells you why you died */
   short           p_whodead;	/* Tells you who killed you */
   struct stats    p_stats;	/* player statistics */
   short           p_genoplanets;	/* planets taken since last genocide */
   short           p_genoarmsbomb;	/* armies bombed since last genocide */
   short           p_planets;	/* planets taken this game */
   short           p_armsbomb;	/* armies bombed this game */
   int             p_ghostbuster;
   int             p_docked;	/* If starbase, # docked to, else pno base
				 * host */
#ifdef SB_DOCK
   int             p_numdocked;	/* remove double meanings of p_docked and */
   int             p_portdocked;/* p_port[0] */
#endif
   int             p_port[4];	/* If starbase, pno of ship docked to that
				 * port, else p_port[0] = port # docked to on
				 * host.   */
   short           p_tractor;	/* What player is in tractor lock */
   int             p_pos;	/* My position in the player file */

   int             p_avrt;	/* average round trip time */
   int             p_stdv;	/* standard deviation in round trip time */
   char            p_pkls_c_s;	/* packet loss (client to server) */
   char            p_pkls_s_c;	/* packet loss (server to client) */

#ifdef BASEPRACTICE
   /* robot ogger variables */
   int		   p_df;	/* defense (0 unknown, 1 worst, 100 best) */
   int		   p_tg;	/* target+1 */
#endif

#ifdef DS
   int		   p_timerdelay;
   int		   p_process;
#endif

#ifdef RCD
    int 	   p_gen_distress;           /* generate generic distress for client */
#endif
};

struct statentry {
   char            name[16], password[16];
   struct stats    stats;
};

/* Torpedo states */

#define TFREE 0
#define TMOVE 1
#define TEXPLODE 2
#define TDET 3
#define TOFF 4
#define TSTRAIGHT 5		/* Non-wobbling torp */

struct torp {
   int             t_no;
   int             t_status;	/* State information */
   int             t_owner;
   int             t_x;
   int             t_y;
   unsigned char   t_dir;	/* direction */
   short           t_turns;	/* rate of change of direction if tracking */
   int             t_damage;	/* damage for direct hit */
   int             t_speed;	/* Moving speed */
   int             t_fuse;	/* Life left in current state */
   char            t_war;	/* enemies */
   char            t_team;	/* launching team */
   char            t_whodet;	/* who detonated... */
#ifdef MINES
   char            t_mine;	/* mine flag */
   int             t_pair;	/* the other torp in the mine pair */
#endif
};

/* Plasma Torpedo states */

#define PTFREE 0
#define PTMOVE 1
#define PTEXPLODE 2
#define PTDET 3

struct plasmatorp {
   int             pt_no;
   int             pt_status;	/* State information */
   int             pt_owner;
   int             pt_x;
   int             pt_y;
   unsigned char   pt_dir;	/* direction */
   short           pt_turns;	/* ticks turned per cycle */
   int             pt_damage;	/* damage for direct hit */
   int             pt_speed;	/* Moving speed */
   int             pt_fuse;	/* Life left in current state */
   char            pt_war;	/* enemies */
   char            pt_team;	/* launching team */
};

#define PHFREE 0x00
#define PHHIT  0x01		/* When it hits a person */
#define PHMISS 0x02
#define PHHIT2 0x04		/* When it hits a photon */

struct phaser {
   int             ph_status;	/* What it's up to */
   unsigned char   ph_dir;	/* direction */
   int             ph_target;	/* Who's being hit (for drawing) */
   int             ph_x, ph_y;	/* For when it hits a torp */
   int             ph_fuse;	/* Life left for drawing */
   int             ph_damage;	/* Damage inflicted on victim */
};

/*
 * An important note concerning planets:  The game assumes that the planets
 * are in a 'known' order.  Ten planets per team, the first being the home
 * planet.
 */

/* the lower bits represent the original owning team */
#define PLREPAIR 0x010
#define PLFUEL 0x020
#define PLAGRI 0x040
#define PLREDRAW 0x080		/* Player close for redraw */
#define PLHOME 0x100		/* home planet for a given team */
#define PLCOUP 0x200		/* Coup has occured */
#define PLCHEAP 0x400		/* Planet was taken from undefended team */
#define PLCORE 0x800		/* A core world planet */

struct planet {
   int             pl_no;
   int             pl_flags;	/* State information */
   int             pl_owner;
   int             pl_x;
   int             pl_y;
   char            pl_name[16];
   int             pl_namelen;	/* Cuts back on strlen's */
   int             pl_armies;
   int             pl_info;	/* Teams which have info on planets */
   int             pl_deadtime;	/* Time before planet will support life */
   int             pl_couptime;	/* Time before coup may take place */
};

#define MVALID 0x01
#define MGOD   0x10  /* this is the biggest MFROM flag - can't overlap this one */

/* order flags by importance (0x100 - 0x400) */
/* restructuring of message flags to squeeze them all into 1 byte - jmn */
/* hopefully quasi-back-compatible:
   MVALID, MINDIV, MTEAM, MALL, MGOD use up 5 bits. this leaves us 3 bits.
   since the server only checks for those flags when deciding message
   related things and since each of the above cases only has 1 flag on at
   a time we can overlap the meanings of the flags */

#define MINDIV 0x02
/* these go with MINDIV flag */
#ifdef STDBG
#define MDBG   0x20
#endif
#define MCONFIG 0x40
#define MDIST  0x60    /* flag distress messages - client thing really but */

#define MTEAM  0x04
/* these go with MTEAM flag */
#define MTAKE  0x20
#define MDEST  0x40
#define MBOMB  0x60
#define MCOUP1 0x80
#define MCOUP2 0xA0
#define MDISTR 0xC0

#define MALL   0x08
/* these go with MALL flag */
#define MGENO  0x20     /* MGENO is not used in INL server but belongs here */
#define MCONQ  0x20     /* not enought bits to distinguish MCONQ from MGENO :-( */
#define MKILLA 0x40
#define MKILLP 0x60
#define MKILL  0x80
#define MLEAVE 0xA0
#define MJOIN  0xC0
#define MGHOST 0xE0
/* MMASK not used in INL server */

#define MWHOMSK  0x1f   /* mask with this to find who msg to */
#define MWHATMSK 0xe0   /* mask with this to find what message about */

struct message {
   int             m_no;
   int             m_flags;
   int             m_time;
   int             m_recpt;
   char            m_data[80];
   int             m_from;
#ifdef SHORT_PACKETS
    int args[8];        /* for SP_S_WARNING short messaging */
#endif
};

/* message control structure */

struct mctl {
   int             mc_current;
};

/* This is a structure used for objects returned by mouse pointing */

#define PLANETTYPE 0x1
#define PLAYERTYPE 0x2

struct obtype {
   int             o_type;
   int             o_num;
};

struct rank {
   float           hours, ratings, defense;
   char           *name;
};

#ifdef VOTE
struct vote {
   
   /* reset galaxy after genocide */
   unsigned int	v_resetgalaxy_votes;
   int	v_resetgalaxy_timer;
   int	v_resetgalaxy;

   /* general reset at any time */
   unsigned int	v_resetplanet_votes;
};
#endif

struct memory {
   struct player   players[MAXPLAYER];
   struct torp     torps[MAXPLAYER * MAXTORP];
   struct plasmatorp plasmatorps[MAXPLAYER * MAXPLASMA];
   struct status   status[1];
   struct planet   planets[MAXPLANETS];
   struct phaser   phasers[MAXPLAYER];
   struct mctl     mctl[1];
   struct message  messages[MAXMESSAGE];
   struct team     teams[MAXTEAM + 1];
#ifdef VOTE
   struct vote	   vote;
#endif
};

#ifdef RSA

#ifndef KEY_SIZE
#define KEY_SIZE        32
#endif

struct rsa_key {
   char		   key_name[2*KEY_SIZE];
   unsigned char   client_type[KEY_SIZE];
   unsigned char   architecture[KEY_SIZE];
   unsigned char   global[KEY_SIZE];
   unsigned char   public[KEY_SIZE];
};
#endif

#ifdef RCD

/* from BRM struct.h */
struct distress {
    unsigned char sender;
    unsigned char dam, shld, arms, wtmp, etmp, fuelp, sts;
    unsigned char wtmpflag, etempflag, cloakflag, distype, macroflag, ttype;
    unsigned char close_pl, close_en, target, tclose_pl, tclose_en, pre_app, i;
    unsigned char close_j, close_fr, tclose_j, tclose_fr;
    unsigned char cclist[6]; /* allow us some day to cc a message
                                up to 5 people */
        /* sending this to the server allows the server to do the cc action */
        /* otherwise it would have to be the client ... less BW this way */
    char preappend[80]; /* text which we pre or append */
};

struct dmacro_list {
    char c;
    char *name;
    char *macro;
};

enum dist_type {
        /* help me do series */
        take=1, ogg, bomb, space_control, help1, help2, help3, help4,

        /* doing series */
        escorting, ogging, bombing, controlling, doing1, doing2, doing3, doing4,

        /* other info series */
        free_beer, /* ie. player x is totally hosed now */
        no_gas, /* ie. player x has no gas */
        crippled, /* ie. player x is way hurt but may have gas */
        pickup, /* player x picked up armies */
        pop, /* there was a pop somewhere */
        carrying, /* I am carrying */
        other1, other2,

        /* just a generic distress call */
        generic };

        /* help1-4, doing1-4, and other1-2 are for future expansion */

/* The General distress has format:

A 'E' will do byte1-8 inclusive. Macros can do more but haven't been
implemented that way yet.

   byte1: xxyzzzzz
        where zzzzz is dist_type, xx is target_type and y is 1 if this is
        a macro and not just a simple distress (a simple distress will ONLY
        send ship info like shields, armies, status, location, etc.)

   byte2: 1fff ffff - f = percentage fuel remaining (0-100)
   byte3: 1ddd dddd - % damage
   byte4: 1sss ssss - % shields remaining
   byte5: 1eee eeee - % etemp
   byte6: 1www wwww - % wtemp
   byte7: 100a aaaa - armies carried
   byte8: (lsb of me->p_status) & 0x80
   byte9: 1ppp pppp - planet closest to me
   byte10: 1eee eeee - enemy closest to me

   Byte 11 and on are only sent if y (from byte 1) is = 1
        although even for simplest case (y=0) we should send byte14+ = 0x80
        and then a null so that in future we can send a cc list and
        pre/append text
   byte11: 1ttt tttt - target (either player number or planet number)
   byte12: 1ppp pppp - planet closest to target
   byte13: 1eee eeee - enemy closest to target
   byte14+: cc list (each player to cc this message to is 11pp ppp)
        cc list is terminated by 1000 0000 or 0100 0000 )
                                 pre-pend     append
   byte15++: the text to pre or append .. depending on termination above.
             text is null terminated and the last thing in this distress
*/

#endif /* RCD */

typedef unsigned char	uchar;
