#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include "defs.h"
#include "struct.h"
#ifndef _NO_DEPEND
#   include "basep_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern pid_t                  wait3 P_((union wait *, int, struct rusage *));
extern int                    shmget P_((key_t, int, int));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void *                 shmat P_((int, void *, int));
extern int                    gethostname P_((char *, int));
extern void                   perror P_((const char *));
extern void                   srandom P_((int));
extern time_t                 time P_((time_t *));
extern int                    tolower P_((int));
extern long                   random P_((void));
extern pid_t                  fork P_((void));
extern int                    execl P_((const char *, const char *, ...));
extern unsigned int           sleep P_((unsigned int));
extern int                    sscanf P_((const char *, const char *, ...));

#undef P_
#endif

struct player  *players;
struct player  *me;
struct torp    *torps;
struct ship    *myship;
struct stats   *mystats;
struct planet  *planets;
struct phaser  *phasers;
struct message *messages;
struct mctl    *mctl;
struct team    *teams;
#ifdef VOTE
struct vote    *vote;
#endif

#define PORT	2592

static char    *team_s[4] = {"federation", "romulan", "klingon", "orion"};

int             shmid;
int             shmemKey = PKEY;

#define NUMADJ 12
static char    *adj_s[NUMADJ] = {
   "VICIOUS", "RUTHLESS", "IRONFISTED", "RELENTLESS",
   "MERCILESS", "UNFLINCHING", "FEARLESS", "BLOODTHIRSTY",
"FURIOUS", "DESPERATE", "FRENZIED", "RABID"};

#define NUMNAMES	20

static char    *names[NUMNAMES] =
{"Annihilator", "Banisher", "Blaster",
   "Demolisher", "Destroyer", "Eliminator",
   "Eradicator", "Exiler", "Obliterator",
   "Razer", "Demoralizer", "Smasher",
   "Shredder", "Vanquisher", "Wrecker",
   "Ravager", "Despoiler", "Abolisher",
"Emasculator", "Decimator"};

/*
 * #define NUMNAMES	3 static char *names[NUMNAMES] = { "guest", "guest",
 * "guest" };
 */

#define RCMD		"/usr/bs/X11R5/bin/xon"
#define OROBOT		"/i0/dh/ua/hadley/alternate/net/robot/og/robot"
#define NICE		"/bin/nice -1"
#define REMOTEHOST	"dv"
#define LOGFILE		"/i0/dh/ua/hadley/alternate/net/robot/og"
#define COMFILE		"/i0/dh/ua/hadley/alternate/net/robot/og/og"
#define DOGFILE		"/i0/dh/ua/hadley/alternate/net/robot/og/dog"
#define BASEFILE	"/i0/dh/ua/hadley/alternate/net/robot/og/base"
#define DEFFILE		"/i0/dh/ua/hadley/alternate/net/robot/og/def"
#define RDOMAIN		"uci"

#define MAX_NUM_PLAYERS	20

char	*orobot		= OROBOT;
char	*_remotehost	= REMOTEHOST;

static char     hostname[64];

void
reaper(sig)
   int	sig;
{
   while (wait3((union wait *) 0, WNOHANG, (struct rusage *) 0) > 0);
}

void
openmem()
{
   extern int      errno;
   struct memory  *sharedMemory;

   errno = 0;
   shmid = shmget(shmemKey, 0, 0);
   if (shmid < 0) {
      if (errno != ENOENT) {
	 fprintf(stderr, "shmget\n");
	 exit(1);
      }
      shmid = shmget(shmemKey, 0, 0);
      if (shmid < 0) {
	 fprintf(stderr, "basep: Daemon not running\n");
	 exit(1);
      }
   }
   sharedMemory = (struct memory *) shmat(shmid, 0, 0);
   if (sharedMemory == (struct memory *) - 1) {
      fprintf(stderr, "shared memory\n");
      exit(1);
   }
   players = sharedMemory->players;
   torps = sharedMemory->torps;
   planets = sharedMemory->planets;
   phasers = sharedMemory->phasers;
   mctl = sharedMemory->mctl;
   messages = sharedMemory->messages;
   teams = sharedMemory->teams;
#ifdef VOTE
   vote = &sharedMemory->vote;
#endif
}

void
main(argc, argv)
   int             argc;
   char           *argv[];
{
   int             robocheck = 0;
   int             oldmctl;
   extern char	   *getenv();

   if (gethostname(hostname, 64) != 0) {
      perror("gethostname");
      exit(1);
   }
   srandom(time(NULL));

   (void) signal(SIGCHLD, reaper);
   openmem();

   if(getenv("OROBOT")){
      orobot = getenv("OROBOT");
   }
   if(getenv("REMOTEHOST")){
      _remotehost = getenv("REMOTEHOST");
   }

   oldmctl = mctl->mc_current;
#ifdef nodef
   for (i = 0; i <= oldmctl; i++) {
      parsemessage(&messages[i].m_data[10], messages[i].m_from);
   }
#endif
   fix_planets();	/* also in doResource'daemonII.c */
   for (;;) {
      sleep(3);
      robocheck++;
      if ((robocheck % 10) == 0)
	 check_robots_only();
      /* make sure shared memory is still valid */
      if (shmget(shmemKey, 0, 0) < 0) {
	 exit(0);
      }
      while (oldmctl != mctl->mc_current) {
	 oldmctl++;
	 if (oldmctl == 50)
	    oldmctl = 0;
	 if (!(messages[oldmctl].m_flags & MALL) &&
	     messages[oldmctl].m_recpt != messages[oldmctl].m_from)
	    continue;

	 parsemessage(&messages[oldmctl].m_data[10],
		      messages[oldmctl].m_from);
      }
   }
}

/* check to see if all robots in the game. If so tell them to exit */

void
check_robots_only()
{
   register        i;
   register struct player *j;
   int             k = 0;

   for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
      if (j->p_status == PFREE)
	 continue;
      if (j->p_flags & PFROBOT)
	 continue;

      k++;

      if (!rprog(j->p_login, j->p_monitor))
	 return;
   }
   if (!k)
      return;

   /*
    * printf("basep: found %d robots, no humans.  Killing..\n", k);
    * fflush(stdout);
    */

   /* all robots */
   for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
      if (j->p_status == PALIVE)
	 pmessage("exit", j->p_no, MINDIV);
   }
}

/* this is by no means foolproof */

int
rprog(login, monitor)
   char           *login, *monitor;
{
   int             v;

   if (strlen(login) < 6 && sscanf(login, "r1h%d", &v) == 1)
      if (strstr(monitor, RDOMAIN))
	 return 1;

   return 0;
}


/* here we want to make sure everything is fuel and repair */

void
fix_planets()
{
   register        i;
   register struct planet *j;
   for (i = 0, j = planets; i < MAXPLANETS; i++, j++)
      j->pl_flags |= (PLREPAIR | PLFUEL);
}

int
num_players()
{
   register        i;
   register struct player *j;
   int             c = 0;
   for (i = 0, j = players; i < MAXPLAYER; i++, j++)
      if (j->p_status != PFREE)
	 c++;
   return c;
}

void
send_help(from)

   int	from;
{
   pmessage(
"To start ROBOT OGGERS, send the following to yourself or all", 
      from, MINDIV);
   pmessage(
"where <number> is the number of oggers and <team> is a 3-letter team name",
      from, MINDIV);
   pmessage(
"start <number> <team>",
      from, MINDIV);

   pmessage(
"-------------------------------------------------------------", 
      from, MINDIV);
   pmessage(
"To start a ROBOT DOGFIGHTER, send the following to yourself or all", 
      from, MINDIV);
   pmessage(
"where <team> is a 3-letter team name",
      from, MINDIV);
   pmessage(
"start <team> df",
      from, MINDIV);
   pmessage(
"-------------------------------------------------------------", 
      from, MINDIV);

   pmessage(
"To start ROBOT SB DEFENDER, send the following to yourself or all", 
      from, MINDIV);
   pmessage(
"where <team> is a 3-letter team name",
      from, MINDIV);
   pmessage(
"start <team> def",
      from, MINDIV);
   pmessage(
"-------------------------------------------------------------", 
      from, MINDIV);

   pmessage(
"To start ROBOT STARBASE, send the following to yourself or all", 
      from, MINDIV);
   pmessage(
"where <team> is a 3-letter team name",
      from, MINDIV);
   pmessage(
"start <team> base",
      from, MINDIV);
   pmessage(
"-------------------------------------------------------------", 
      from, MINDIV);
   pmessage(
"See the MOTD for specific robot commands.",
      from, MINDIV);
}

void
parsemessage(s, from)
   char           *s;
   int             from;
{
   register        i;
   int             sv;
   char            buf[80], team[80], host[80], log[80], extra[80], desc[80];
   int             num;
   int             teamv = 0, teami = -1;
   int             dg = 0, base = 0, def = 0;

   while (isspace(*s))
      s++;
   strcpy(buf, s);

   for (i = 0; i < strlen(s); i++)
      if (isupper(buf[i]))
	 buf[i] = tolower(buf[i]);
   
   if(strcmp(buf, "help") == 0){
      send_help(from);
      return;
   }

   sv = sscanf(buf, "start %d %s %s %s", &num, team, host, log);
   if (sv < 2) {
      sv = sscanf(buf, "send %d %s %s %s", &num, team, host, log);
      if (sv < 2) {
	 sv = sscanf(buf, "gimme %d %s %s %s", &num, team, host, log);
	 if (sv < 2) {
	    sv = sscanf(buf, "start %s %s", team, desc);
	    if (sv == 2) {
	       if (strcmp(desc, "df") == 0) {
		  num = 1;
		  dg = 1;
	       } else if (strcmp(desc, "base") == 0) {
		  num = 1;
		  base = 1;
	       } else if (strcmp(desc, "def") == 0) {
		  num = 1;
		  def = 1;
	       } else
		  return;
	    } else
	       return;
	 }
      }
   }
   if (strncmp(team, "fed", 3) == 0) {
      teamv = 0;
      teami = FED;
      strcpy(team, "-Tf");
   } else if (strncmp(team, "rom", 3) == 0) {
      teamv = 1;
      teami = ROM;
      strcpy(team, "-Tr");
   } else if (strncmp(team, "kli", 3) == 0) {
      teamv = 2;
      teami = KLI;
      strcpy(team, "-Tk");
   } else if (strncmp(team, "ori", 3) == 0) {
      teamv = 3;
      teami = ORI;
      strcpy(team, "-To");
   } else {
      sprintf(buf, "Unknown team name \"%s\"", team);
      pmessage(buf, from, MINDIV);
      return;
   }

   if (num_players() + num > MAX_NUM_PLAYERS) {
      pmessage("Too many players, sorry.", from, MINDIV);
      return;
   }
#ifdef nodef
   if (teami == players[from].p_team) {
      sprintf(buf, "Wrong team, pal.");
      pmessage(buf, from, MINDIV);
      return;
   }
#endif

   extra[0] = '\0';

   if (sv >= 3) {
      strcpy(extra, " (");
      if (sv > 3) {
	 strcat(extra, host);
	 strcat(extra, " log");
      } else{
	 strcat(extra, "on remote host\"");
	 strcat(extra, host);
	 strcat(extra, "\"");
      }
      strcat(extra, ")");
   }
   if (dg)
      sprintf(buf, "%d %s %s dogfighter%s started.%s", num,
	      adj_s[random() % NUMADJ],
	      team_s[teamv], num == 1 ? "" : "s", extra);
   else if (base)
      sprintf(buf, "one moderately angry %s base started.%s",
	      team_s[teamv], extra);
   else if (def)
      sprintf(buf, "one barely competent %s defender started.%s",
	      team_s[teamv], extra);
   else
      sprintf(buf, "%d %s %s ogger%s started.%s", num,
	      adj_s[random() % NUMADJ],
	      team_s[teamv], num == 1 ? "" : "s", extra);


   pmessage(buf, 0, MALL);

   startrobot(num, team, sv >= 3 ? host : NULL, sv > 3 ? log : NULL, dg, base, def);
}

char           *
namearg()
{
   register        i, k = 0;
   register struct player *j;
   char           *name;
   int             namef = 1;

   while (1) {

      name = names[random() % NUMNAMES];
      k++;

      namef = 0;
      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if (j->p_status != PFREE && strncmp(name, j->p_name, strlen(name) - 1)
	     == 0) {
	    namef = 1;
	    break;
	 }
      }
      if (!namef)
	 return name;

      if (k == 50)
	 return "guest";
   }
}

int
startrobot(num, s, h, log, dg, base, def)
   int             num;
   char           *s, *h, *log;
   int             dg, base, def;
{
   char           *remotehost;
   char            command[256];
   char            logc[256];
   register        i;

   if (h)
      remotehost = h;
   else
      remotehost = _remotehost;
   if (log)
      sprintf(logc, "-l %s", LOGFILE);
   else
      sprintf(logc, " ");

   for (i = 0; i < num; i++) {
      if (dg)
	 sprintf(command, "%s %s %s %s -h %s -p %d -n '%s' -X r1h1f -b -O %s -C %s",
	       RCMD, remotehost, orobot, s, hostname, PORT, namearg(), logc,
		 DOGFILE);
      else if (base)
	 sprintf(command, "%s %s %s %s -h %s -p %d -n 'RoboBase' -X r1h1 -co -b -O %s -C %s",
		 RCMD, remotehost, orobot, s, hostname, PORT, logc,
		 BASEFILE);
      else if (def)
	 sprintf(command, "%s %s %s %s -h %s -p %d -n 'Defender' -X r1h1d -b -O %s -C %s",
		 RCMD, remotehost, orobot, s, hostname, PORT, logc,
		 DEFFILE);
      else
	 sprintf(command, "%s %s %s %s %s -h %s -p %d -n '%s' -X r1h0o -S -g -b -O %s -C %s",
		 RCMD, remotehost, NICE, orobot, s, hostname, PORT,
		 namearg(), logc, COMFILE);

      /*
       * printf("%s\n", command);
       */

      if (fork() == 0) {
	 execl("/bin/sh", "sh", "-c", command, 0);
	 perror("basep'execl");
	 _exit(1);
      }
      sleep(5);
   }
   return 1;
}

void
pmessage(str, recip, group)
   char           *str;
   int             recip;
   int             group;
{
   static char     buf[160];
   struct message *cur;

   if (group == MALL)
      sprintf(buf, "GOD->ALL  %s", str);
   else
      sprintf(buf, "GOD->%c%c   %s", players[recip].p_mapchars[0],
	      players[recip].p_mapchars[1], str);
   if (++(mctl->mc_current) >= MAXMESSAGE)
      mctl->mc_current = 0;
   cur = &messages[mctl->mc_current];
   cur->m_no = mctl->mc_current;
   cur->m_flags = group;
   cur->m_time = 0;
   cur->m_recpt = recip;
   cur->m_from = 255;		/* change 12/11/90 TC */
   strncpy(cur->m_data, buf, 79);
   cur->m_data[79] = '\0';
   cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
   cur->args[0] = DINVALID; /* to prevent false sending with SPS_S_WARNING*/
#endif
}

#ifdef hpux
int
srandom(foo)
   int             foo;
{
   rand(foo);
}

int
random()
{
   return (rand());
}

int
nint(x)
   double          x;
{
   return (int) x;		/* xx */
}
#endif
