/*
 * birdmove.c
 */
#include "copyright.h"

#include <stdio.h>
#include <signal.h>
#include <values.h>
#include <math.h>
#include <time.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#ifndef _NO_DEPEND
#include "bird_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern long                   random P_((void));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    printf P_((const char *, ...));
extern void                   bzero P_((void *, size_t));
extern int                    fflush P_((FILE *));
extern void                   perror P_((const char *));

#undef P_
#endif

#define WINGSPAN	300

/* when to do random coasting */
#define COASTCHECK	20

#define DAMAGESPINTIME	30
#define DIETIME		30

#define MAXDAM		3

#define PHASERFUSE	2

#define PHASERDAM	20

#define HEALCOUNT	50	/* 1 per 5 seconds */

/* when speed changes */
#define SPEED_INC	7
#define SPEED_DEC	10

/* threshold to avoid galactic boundaries */
#define MD		4500
/* overlap threshold on team boundaries */
#define TMD		2250

/* how often to check for bombers in cyles (1/10th second) */
#define BOMBCHECK	20

/* how often to check for players in the game in cyles (1/10th second) */
#define PLAYERCHECK	100

#define WARHOSTILE(p)		((((p)->p_swar | (p)->p_hostile)& \
				perfs[0]->p_team) || ((perfs[0]->p_swar | \
				perfs[0]->p_hostile) & (p)->p_team))

extern struct player *perfs[2];
extern int      num_perfs;
extern int      debug;
extern int      lastm;
extern int      tno;
extern int      target;
extern int      berserk;
extern int      patrol;
extern int      team1, team2;
extern int      agri_home;

static int      b_clock;
static          _move();
static int      defenders[ALLTEAM];
static int      at_agri;
static int      damagespin;
static int      die;
static int      damcount;
static int      heal;
static int      maxdam;

unsigned char   getacourse();
struct planet  *nearest_ag();

void
birdmove()
{
   b_clock++;

   _move();
}

static int
_move()
{
   if (!perfs[0] || !perfs[1])
      exitBird(0);

   if (perfs[0]->p_status == PFREE || perfs[1]->p_status == PFREE)
      exitBird(0);

   /* keep ghostbuster away */
   perfs[0]->p_ghostbuster = 0;
   perfs[1]->p_ghostbuster = 0;

   /*
    * the Bird explodes upon entering game and remains in this state
    * throughout
    */

   perfs[0]->p_status = PDEAD;
   perfs[1]->p_status = PDEAD;
   perfs[0]->p_explode = 100;
   perfs[1]->p_explode = 100;

   if (b_clock == 5)
      startbird();
   else if (b_clock > 5) {
      if (perfs[0]->p_damage > 0 || perfs[1]->p_damage > 0 || damagespin || die) {
	 damagebird();
	 if (die == 1) {
	    die = 0;
	    b_clock = -20;	/* xx */
	 }
      } else
	 movebird();
   }
   return 1;
}

void
startbird()
{
   struct player  *p1 = perfs[0], *p2 = perfs[1];
   int             startx, starty;

   damcount = 0;

   startx = planets[tno * 10].pl_x + (random() % 10000) - 5000;
   starty = planets[tno * 10].pl_y + (random() % 10000) - 5000;

   p1->p_x = startx;
   p1->p_y = starty;

   p2->p_x = startx;
   p2->p_y = starty;

   /* left wing p1 */
   birdphaser(p1, 192);
   /* right wing p2 */
   birdphaser(p2, 64);

   if (debug)
      fprintf(stderr, "started\n");

   if (target > -1)
      maxdam = 3 * MAXDAM;
   else
      maxdam = MAXDAM;
}

static int      birdspeed = 1;
static int      birdspeed_frac = 0;
static int      birdwingangle = 64;
static int      birdwingflapup = 12;
static int      birdwingflapdown = 16;
static int      birdwingflap = 10;
static int      birdcoast = 0;
static int      birdcoastwingangle = 72;
static int      birdroosttime = 50;
static int      birdperch = 0;
static int      birdroost = 0;

void
movebird()
{
   register        i;
   register struct player *j;
   unsigned char   tc;
   struct player  *p1 = perfs[0], *p2 = perfs[1], *tr;
   int             dir = 6;
   int             ok;
   unsigned char   dirl, dirr;
   static
   int             damc = 0;

   if ((b_clock % PLAYERCHECK) == 0) {
      /*
       * every x seconds make sure there's people in the game
       */
      ok = 0;
      defenders[FED] = defenders[ROM] = defenders[KLI] = defenders[ORI] = 0;
      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if ((j->p_status != PFREE) && !(j->p_flags & PFROBOT)) {
	    ok = 1;
	    defenders[j->p_team]++;
	 }
      }
      if (!ok)
	 exitBird(0);
   }
   if (patrol && (target == -1) && (b_clock % BOMBCHECK) == 0) {
      target = bombcheck(team1, team2);
   }
   if ((b_clock % (4 + (random() % 4))) == 0)
      dir = -dir;

   if (birdspeed > 10 || !birdcoast)
      dir = dir > 0 ? 3 : -3;
   else
      dir = dir > 0 ? 6 : -6;

   if (!birdroost && birdspeed > 2) {
      p1->p_dir += dir + rrnd(4);

      if (target > -1) {
	 tr = &players[target];
	 if (tr->p_status == PALIVE) {
	    int             nd, td;
	    tc = getacourse(tr->p_x, tr->p_y, p1->p_x, p1->p_y);
	    nd = angdist(p1->p_dir, tc);
	    if (nd > 8) {
	       td = tc + nd;
	       if (td == p1->p_dir)
		  p1->p_dir -= 8;
	       else
		  p1->p_dir += 8;
	    }
	    if ((b_clock % 5) == 0)
	       damc = dodamage(tr, &phasers[p1->p_no],
			       &phasers[p2->p_no]);
	 } else
	    exitBird(0);
      } else if (agri_home) {
	 int             a_dist;
	 struct planet  *a = nearest_ag(p1, &a_dist);
	 at_agri = 0;
	 if (!a)
	    agri_home = 0;
	 else if (a_dist > 7000) {
	    int             nd, td;
	    tc = getacourse(a->pl_x, a->pl_y, p1->p_x, p1->p_y);
	    nd = angdist(p1->p_dir, tc);
	    if (nd > 8) {
	       td = tc + nd;
	       if (td == p1->p_dir)
		  p1->p_dir -= 8;
	       else
		  p1->p_dir += 8;
	    }
	 } else
	    at_agri = 1;
      } else {
	 if (!patrol) {
	    check_tboundary(ALLTEAM, &p1->p_dir, 4, p1->p_x, p1->p_y);
	 } else {
	    check_tboundary(team1 | team2, &p1->p_dir, 4, p1->p_x, p1->p_y);
	 }
      }
   }
   /* NOTE: the daemon won't move either player because they are PFDEAD */

   p1->p_x += (double) (birdspeed * WARP1) * Cos[p1->p_dir];
   p1->p_y += (double) (birdspeed * WARP1) * Sin[p1->p_dir];

   /* always on top of each other */
   p2->p_dir = p1->p_dir;
   p2->p_x = p1->p_x;
   p2->p_y = p1->p_y;

   if (!birdcoast && !birdperch && (b_clock % COASTCHECK) == 0) {
      if (((random() % 3) == 0 || birdspeed > 10) && birdspeed > 3) {
	 birdcoast = 10 + random() % (10 + birdspeed);
	 birdcoastwingangle = 64 + random() % 32;
      }
      if ((random() % (!at_agri ? 10 : 5)) == 1) {
	 birdperch = 1;
	 birdroost = 0;
	 birdroosttime = at_agri ? (50 + random() % 600) : 50 + random() % 30;
	 if (target > -1)
	    birdroosttime = 20;
      }
   }
   if (target > -1 && birdcoast && birdspeed < 7)
      birdcoast = 0;

   if (birdcoast && !birdperch) {

      /* coast */
      if (ABS(birdwingangle - birdcoastwingangle) > 8) {
	 if (birdwingangle < birdcoastwingangle)
	    birdwingangle += 8;
	 else
	    birdwingangle -= 8;
      }
      /*
       * birdwingangle = birdcoastwingangle;
       */
      if (debug)
	 printf("coast at %d\n", birdcoastwingangle);

      birdspeed_frac--;
      if (birdspeed_frac <= 0) {
	 birdspeed--;
	 if (birdspeed < 1)
	    birdspeed = 1;
	 birdspeed_frac = SPEED_DEC;
      }
      if (birdcoast > 0)
	 birdcoast--;
   } else if (birdspeed > 0 && !birdperch) {

      /* higher flap rate for slower speeds */
      birdwingflapdown = 20 + (10 - birdspeed);
      if (birdspeed < 4)
	 birdwingflapdown += 10;
      birdwingflapup = birdwingflapdown - 8;
      if (birdwingflapup < 0)
	 birdwingflapup = 4;

      if (birdwingangle <= 50)
	 birdwingflap = birdwingflapdown;
      else if (birdwingangle >= 100)
	 birdwingflap = -birdwingflapup;

      birdwingangle += birdwingflap;

      if (debug)
	 printf("flap at inc %d\n", birdwingflap);

      if (birdspeed < 5)
	 birdspeed_frac += 2;
      else
	 birdspeed_frac++;
      if (birdspeed_frac >= SPEED_INC) {
	 birdspeed++;
	 if (birdspeed > 14)
	    birdspeed = 14;
	 birdspeed_frac = 0;
      }
   } else {
      birdspeed_frac -= 2;
      if (birdspeed_frac <= 0) {
	 birdspeed--;
	 if (birdspeed < 0)
	    birdspeed = 0;
	 birdspeed_frac = SPEED_DEC;
      }
      if (birdspeed < 2) {
	 if (p1->p_dir > 128)
	    p1->p_dir += (256 - p1->p_dir) / 2;
	 else
	    p1->p_dir -= p1->p_dir / 2;
      }
      if (birdspeed == 0) {
	 if (debug)
	    printf("bird roost\n");

	 if (ABS(birdwingangle - 16) > 12) {
	    if (birdwingangle < 16)
	       birdwingangle += 12;
	    else
	       birdwingangle -= 12;
	 } else
	    birdroost = 1;

	 if (birdroost) {
	    birdwingangle = 16;
	    p1->p_dir = 0;
	    p2->p_dir = 0;
	    if (--birdroosttime <= 0) {
	       birdroosttime = 0;
	       birdspeed = 1;
	       birdperch = 0;
	       birdroost = 0;
	    }
	    heal++;
	    if (heal == HEALCOUNT) {
	       if (damcount)
		  damcount--;
	       heal = 0;
	    }
	 }
      } else if (debug)
	 printf("bird perch\n");
   }

   /*
    * printf("birdwingangle: %d\n", birdwingangle);
    */

   dirl = p1->p_dir - (unsigned char) birdwingangle;
   dirr = p1->p_dir + (unsigned char) birdwingangle;

   /*
    * printf("dir left wing: %d (%d)\n", dirl, p1->p_dir); printf("dir right
    * wing: %d (%d)\n", dirr, p1->p_dir);
    */

   if (target > -1) {
      if (damc == 0) {
	 birdphaser(p1, dirl);
	 birdphaser(p2, dirr);
      } else {
	 damc--;
      }
   } else {
      birdphaser(p1, dirl);
      birdphaser(p2, dirr);
   }
}

void
damagebird()
{
   struct player  *p1 = perfs[0], *p2 = perfs[1];
   static struct player *whofired;
   int             dam;
   static unsigned char spinc;
   unsigned char   dirl, dirr;
   register        x, y;
   struct phaser  *ph;

   if (!die) {
      if (p1->p_damage > 0) {
	 whofired = &players[p1->p_whodead];
	 dam = p1->p_damage;
	 p1->p_whodead = p1->p_no;	/* xx */
	 p1->p_damage = 0;
	 spinc = getacourse(whofired->p_x, whofired->p_y, p1->p_x, p1->p_y);
	 spinc += 128;

	 damcount++;
	 if (damcount < maxdam)
	    damagespin = DAMAGESPINTIME;
	 else
	    die = DIETIME;
      } else if (p2->p_damage > 0) {
	 whofired = &players[p2->p_whodead];
	 dam = p2->p_damage;
	 p2->p_whodead = p2->p_no;	/* xx */
	 p2->p_damage = 0;
	 spinc = getacourse(whofired->p_x, whofired->p_y, p1->p_x, p1->p_y);
	 spinc += 128;
	 damcount++;
	 if (damcount < 3)
	    damagespin = DAMAGESPINTIME;
	 else
	    die = DIETIME;
      }
      birdspeed = 1;
      birdperch = 0;
      birdroost = 0;
      birdroosttime = 0;

      /* fall through if hit twice */
   }
   if (damagespin > 0) {

      if (debug)
	 printf("damaged and spinning\n");

      damagespin--;

      p1->p_x += (double) (((20 * damagespin) / DAMAGESPINTIME) * WARP1) * Cos[spinc];
      p1->p_y += (double) (((20 * damagespin) / DAMAGESPINTIME) * WARP1) * Sin[spinc];
      p1->p_dir += (64 * damagespin) / DAMAGESPINTIME;

      p2->p_x = p1->p_x;
      p2->p_y = p1->p_y;
      p2->p_dir = p1->p_dir;

      dirl = p1->p_dir + 64;
      dirr = p2->p_dir - 32;

      birdphaser(p1, dirl);
      birdphaser(p2, dirr);
   } else if (die >= 0) {

      if (debug)
	 printf("dieing\n");

      die--;

      ph = &phasers[p1->p_no];

      /* the new point given is the center of the phaser wing */
      x = p1->p_x + (WINGSPAN / 2) * Cos[ph->ph_dir];
      y = p1->p_y + (WINGSPAN / 2) * Sin[ph->ph_dir];

      /* move this point */
      x += (double) (((30 * die) / DIETIME) * WARP1) * Cos[spinc - 32];
      y += (double) (((30 * die) / DIETIME) * WARP1) * Sin[spinc - 32];

      dirl = ph->ph_dir + (76 * die) / DIETIME;

      /* and then recalculate player position */
      p1->p_x = x - (WINGSPAN / 2) * Cos[dirl];
      p1->p_y = y - (WINGSPAN / 2) * Sin[dirl];

      ph = &phasers[p2->p_no];

      x = p2->p_x + (WINGSPAN / 2) * Cos[ph->ph_dir];
      y = p2->p_y + (WINGSPAN / 2) * Sin[ph->ph_dir];

      x += (double) (((30 * die) / DIETIME) * WARP1) * Cos[spinc + 32];
      y += (double) (((30 * die) / DIETIME) * WARP1) * Sin[spinc + 32];

      dirr = ph->ph_dir + (70 * die) / DIETIME;

      p2->p_x = x - (WINGSPAN / 2) * Cos[dirr];
      p2->p_y = y - (WINGSPAN / 2) * Sin[dirr];

      p1->p_dir += (76 * die) / DIETIME;
      p2->p_dir += (70 * die) / DIETIME;

      birdphaser(p1, dirl);
      birdphaser(p2, dirr);
   }
}

void
exitBird(sig)
   int             sig;
{
   register int    i;
   register struct player *j;
   register struct torp *k;
   register struct plasmatorp *f;

   signal(SIGALRM, SIG_IGN);

   if (debug)
      fprintf(stderr, "bird exiting\n");

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {
	 /* torps are free here */
	 if (k->t_status != TFREE)
	    k->t_status = TOFF;
      }
      f = &plasmatorps[j->p_no * MAXPLASMA];
      f->pt_status = PTFREE;
      /* plasma */
   }
   bzero(&phasers[perfs[0]->p_no], sizeof(struct phaser));
   bzero(&phasers[perfs[1]->p_no], sizeof(struct phaser));

   bzero(perfs[0], sizeof(struct player));
   bzero(perfs[1], sizeof(struct player));
   perfs[0]->p_stats.st_tticks = 1;
   perfs[1]->p_stats.st_tticks = 1;

   /*
    * note: if we had a segmentation violation or bus error we want a core
    * file to debug
    */
   if (sig == SIGBUS || sig == SIGSEGV)
      abort();
   else
      exit(0);
}

void
pmessage(str, recip, group, address)
   char           *str;
   int             recip;
   int             group;
   char           *address;
{
   pmessage2(str, recip, group, address, 255);
}

void
pmessage2(str, recip, group, address, from)
   char           *str;
   int             recip;
   int             group;
   char           *address;
   unsigned char   from;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = from;
   (void) sprintf(cur->m_data, "%-9s %s", address, str);
   cur->m_flags |= MVALID;
}

/* get course from (mx,my) to (x,y) */
unsigned char 
getacourse(x, y, mx, my)
   int             x, y, mx, my;
{
   unsigned char	d;
   if (x == mx && y == my)
      return 0;

   d = (unsigned char) (atan2((double) (x - mx),
				 (double) (my - y)) / 3.14159 * 128.);
   MATH_ERR_CHECK(x-mx+my-y);
   return d;
}

/*
 * FED 1 ROM 2 KLI 4 ORI 8
 */

static struct {
   int             lx, rx, ty, by;
}               boundaries[ALLTEAM + 1] = {
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 0    */
   {
      MD, GWIDTH / 2 + TMD, GWIDTH / 2 - TMD, GWIDTH - MD
   },				/* 1 FED */
   {
      MD, GWIDTH / 2 + TMD, MD, GWIDTH / 2 + TMD
   },				/* 2 ROM */
   {
      MD, GWIDTH / 2 + TMD, MD, GWIDTH - MD
   },				/* 3 FED|ROM */
   {
      GWIDTH / 2 - TMD, GWIDTH - MD, MD, GWIDTH / 2 + TMD
   },				/* 4 KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 5 FED|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH / 2 + TMD
   },				/* 6 ROM|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 7 */
   {
      GWIDTH / 2 - TMD, MD, GWIDTH / 2 - TMD, MD
   },				/* 8 ORI */
   {
      MD, GWIDTH - MD, GWIDTH / 2 - TMD, GWIDTH - MD
   },				/* 9 ORI|FED */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 10 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 11 */
   {
      GWIDTH / 2 - TMD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 12 ORI|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 13 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 14 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 15 */
};

/*
 * 256/0 192  +   64 128 */

void
check_tboundary(teams, mycrs, range, x, y)
   int             teams;
   int             x, y;
   unsigned char  *mycrs;
   int             range;
{
   int             lx = boundaries[teams].lx, rx = boundaries[teams].rx, ty = boundaries[teams].ty, by = boundaries[teams].by;
   int             r = 0, l = 0;
   unsigned char   crs = *mycrs;

   if (x < lx && crs >= 128) {
      if (crs >= 192)
	 *mycrs += range;
      else
	 *mycrs -= range;
      l = 1;
   } else if (x > rx && crs < 128) {
      if (crs > 64)
	 *mycrs += range;
      else
	 *mycrs -= range;
      r = 1;
   }
   if (y < ty && (crs >= 192 || crs < 64)) {
      if (crs >= 192 && !l)
	 *mycrs -= range;
      else
	 *mycrs += range;
   } else if (y > by && (crs < 192 && crs >= 64)) {
      if (crs < 128 && !r)
	 *mycrs -= range;
      else
	 *mycrs += range;
   }
}

/* return rnd between -range & range */
int
rrnd(range)
   int             range;
{
   return random() % (2 * range) - range;
}

int
bombcheck(team1, team2)
   int             team1, team2;
{
   register        i;
   register struct player *j;
   struct planet  *p;

   for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
      if (j->p_status == PALIVE) {
	 if ((j->p_flags & PFBOMB) && (j->p_flags & PFORBIT)) {
	    p = &planets[j->p_planet];
	    if (!((j->p_swar | j->p_hostile) & p->pl_owner))
	       continue;

	    if (defenders[p->pl_owner] >= tournplayers)
	       continue;

	    /* ignore planets that aren't within team boundaries */
	    if (p->pl_x > boundaries[p->pl_owner].rx ||
		p->pl_x < boundaries[p->pl_owner].lx ||
		p->pl_y > boundaries[p->pl_owner].by ||
		p->pl_y < boundaries[p->pl_owner].ty)
	       continue;

	    if (!team1 && !team2) {
	       /* any planet */
	       printf("bird found bomber: targeting %c%c\n",
		      teamlet[j->p_team], shipnos[j->p_no]);
	       fflush(stdout);
	       make_war(j, p->pl_owner);
	       return j->p_no;
	    } else {
	       if (team1) {
		  if (p->pl_owner == team1) {
		     printf("found bomber: targeting %c%c\n",
			    teamlet[j->p_team], shipnos[j->p_no]);
		     fflush(stdout);
		     make_war(j, p->pl_owner);
		     return j->p_no;
		  }
	       }
	       if (team2) {
		  if (p->pl_owner == team2) {
		     printf("found bomber: targeting %c%c\n",
			    teamlet[j->p_team], shipnos[j->p_no]);
		     fflush(stdout);
		     make_war(j, p->pl_owner);
		     return j->p_no;
		  }
	       }
	    }
	 }
      }
   }
   return -1;
}

void
make_war(p, plteam)
   struct player  *p;
   int             plteam;
{
   register int    i;
   register struct player *j;
   register struct torp *k;

   perfs[0]->p_team = plteam;
   perfs[0]->p_swar = 0;
   perfs[0]->p_hostile = 0;
   perfs[1]->p_team = plteam;
   perfs[1]->p_swar = 0;
   perfs[1]->p_hostile = 0;

   perfs[0]->p_swar |= p->p_team;
   perfs[0]->p_hostile |= p->p_team;
   perfs[1]->p_swar |= p->p_team;
   perfs[1]->p_hostile |= p->p_team;

   /* update our torps war status */
   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {

	 k->t_war = perfs[0]->p_swar | perfs[0]->p_hostile;
      }
   }
}



/*
 * start planet areas are places where people will likely enter the game
 * (only considers 1 start planet at this time)
 */

int
sp_area(x, y)
   int             x, y;
{
   register        i, j, px, py;
   for (i = 0; i < NUMTEAM; i++) {
      for (j = 0; j < 10; j++)	/* default executes only once */
	 if (startplanets[i * j])
	    break;
      px = planets[i * 10 + j].pl_x;
      py = planets[i * 10 + j].pl_y;
      /*
       * printf("checking %s (%d,%d)\n", planets[i*10+j].pl_name, i*10,j);
       */

      if (!(x < px - 5300 || x > px + 5300 || y < py - 5300 || y > py + 5300)) {
	 return 1;
      }
   }
   return 0;
}

void
birdphaser(p, dir)
   struct player  *p;
   unsigned char   dir;
{
   register struct phaser *mine;
   int             ex, ey;

   ex = p->p_x + WINGSPAN * Cos[dir];
   ey = p->p_y + WINGSPAN * Sin[dir];

   mine = &phasers[p->p_no];
   mine->ph_dir = dir;
   mine->ph_fuse = 10;
   mine->ph_status = PHHIT2;	/* so end coordinates included */
   mine->ph_x = ex;
   mine->ph_y = ey;
   if (!birdroost)
      mine->ph_target++;	/* xx: force update from server */
}

struct planet  *
nearest_ag(j, a_dist)
   struct player  *j;
   int            *a_dist;
{
   static struct planet *na;	/* cache */
   register        i, d;
   register struct planet *k, *curr = NULL;
   int             mindist = 32767;

   if (!na) {
      for (i = 0, k = planets; i < MAXPLANETS; i++, k++) {
	 if (k->pl_flags & PLAGRI) {
	    d = ihypot(j->p_x - k->pl_x, j->p_y - k->pl_y);
	    if (d < mindist) {
	       mindist = d;
	       curr = k;
	    }
	 }
      }
      if (curr) {
	 na = curr;
	 *a_dist = mindist;
      }
   } else {
      d = ihypot(j->p_x - na->pl_x, j->p_y - na->pl_y);
      *a_dist = d;
   }
   return na;
}

/* damage target */

int
dodamage(tr, ph1, ph2)
   struct player  *tr;
   struct phaser  *ph1, *ph2;
{
   int             rx1 = tr->p_x - ph1->ph_x, ry1 = tr->p_y - ph1->ph_y, rx2 = tr->p_x - ph2->ph_x, ry2 = tr->p_y - ph2->ph_y;
   int             hit = 0;

   if (ihypot(rx1, ry1) < EXPDIST) {
      ph1->ph_status = PHHIT;
      ph1->ph_fuse = PHASERFUSE;
      ph1->ph_damage = PHASERDAM;
      ph1->ph_target = tr->p_no;
      hit = 1;
      /*
       * printf("hit phase 1\n");
       */
   }
   if (ihypot(rx2, ry2) < EXPDIST) {
      ph2->ph_status = PHHIT;
      ph2->ph_fuse = PHASERFUSE;
      ph2->ph_damage = PHASERDAM;
      ph2->ph_target = tr->p_no;
      hit = 1;
      /*
       * printf("hit phase 2\n");
       */
   }
   if (hit)
      return PHASERFUSE;
   else
      return 0;
}

/*
 * gives approximate distance from (x1,y1) to (x2,y2) with only
 * overestimations, and then never by more than (9/8) + one bit uncertainty.
 */

int 
ihypot(x1, y1)
   int             x1, y1;
{
   register        x2 = 0, y2 = 0;

   if ((x2 -= x1) < 0)
      x2 = -x2;
   if ((y2 -= y1) < 0)
      y2 = -y2;
   return (x2 + y2 - (((x2 > y2) ? y2 : x2) >> 1));
}
