/*
 * borg.c
 */
#include "copyright.h"

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#ifndef _NO_DEPEND
#include "borg_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   srandom P_((int));
extern pid_t                  getpid P_((void));
extern time_t                 time P_((time_t *));
extern void                   bzero P_((void *, size_t));
extern int                    fprintf P_((FILE *, const char *, ...));
extern char *                 strncpy P_((char *, const char *, size_t));
extern int                    setitimer P_((int, struct itimerval *, struct itimerval *));
extern void                   perror P_((const char *));
extern int                    sigsetmask P_((int));
extern int                    pause P_((void));
extern int                    printf P_((const char *, ...));
extern unsigned int           sleep P_((unsigned int));

#undef P_
#endif

int		nukepl[MAXPLANETS];
int		num_nukepl = 0;
int		updates = 2;
int             debug = 0;

void
main(argc, argv)
   int             argc;
   char          **argv;
{
   register int    i;
   int             class = STARBASE, pno;

   srandom(getpid() * time(NULL));
   bzero(nukepl, sizeof(nukepl));

   for (; argc > 1 && argv[1][0] == '-'; argc--, argv++) {
      switch (argv[1][1]) {
      case 'd':
	 debug++;
	 break;
      default:
	 fprintf(stderr, "Unknown option '%c' (must be one of: dTc)\n",
		 argv[1][1]);
	 exit(1);
      }				/* end switch argv[1][1] */
   }				/* end for */
   readsysdefaults();

   openmem();

   if(argc == 1){
      fprintf(stderr, "borg: No planets specified on command line\n");
      exit(1);
   }
   for(i=1; i< argc; i++){
      nukepl[i-1] = plname_to_index(argv[i]);
      if(nukepl[i-1] < 0){
	 fprintf(stderr, "borg: Unknown planet \"%s\"\n", argv[i]);
	 exit(1);
      }
   }
   num_nukepl = argc-1;

   if (!killrobots()) {
      fprintf(stderr, "borg: Couldn't kill robots\n");
      exit(1);
   }

   (void) signal(SIGHUP, exitBorg);
   (void) signal(SIGTERM, exitBorg);
   (void) signal(SIGINT, exitBorg);
   (void) signal(SIGBUS, exitBorg);
   (void) signal(SIGSEGV, exitBorg);

   pno = findrslot();
   if (pno < 0) {
      fprintf(stderr, "borg: no room in game\n");
      exit(1);
   }
   me = &players[pno];
   me->p_no = pno;
   myship = &me->p_ship;
   mystats = &me->p_stats;

   strcpy(pseudo, "BORG");
   strcpy(login, "robot");
   strcpy(me->p_name, pseudo);
   me->p_name[sizeof(me->p_name) - 1] = '\0';

   (void) strncpy(me->p_login, login, sizeof(me->p_login));
   me->p_login[sizeof(me->p_login) - 1] = '\0';
   (void) strncpy(me->p_monitor, "Server", sizeof(me->p_monitor));
   me->p_monitor[sizeof(me->p_monitor) - 1] = '\0';

   borgenter(NOBODY, pno, class);
   shipconfig();

   me->p_pos = -1;
   me->p_flags |= PFROBOT;	/* Mark as a robot */
   me->p_hostile = ALLTEAM;

   signal(SIGALRM, borgmove);

#ifndef DS
   udt.it_interval.tv_sec = 0;
   udt.it_interval.tv_usec = 100000 * updates;
   udt.it_value.tv_sec = 1;
   udt.it_value.tv_usec = 0;

   if (setitimer(ITIMER_REAL, &udt, 0) < 0) {
      perror("setitimer");
      for (i = 0; i < NUMTEAM; i++) {
	 if (perfs[i])
	    perfs[i]->p_status = PFREE;
      }
      exit(1);
   }
#else
   me->p_process = getpid();
   me->p_timerdelay = updates;
#endif

   /* allows robots to be forked by the daemon -- Evil ultrix bullshit */
   sigsetmask(0);

   me->p_status = PALIVE;

   while (1) {
      pause();
   }
}

int
findtestslot()
{
   register int    i;

   for (i = MAXPLAYER - TESTERS; i < MAXPLAYER; i++) {
      if (players[i].p_status == PFREE) {	/* We have a free slot */
	 players[i].p_status = POUTFIT;	/* possible race condition */
	 break;
      }
   }
   if (i == MAXPLAYER) {
      return -1;		/* no room in tester slots */
   }
   bzero(&players[i].p_stats, sizeof(struct stats));
   players[i].p_stats.st_tticks = 1;
   return (i);
}

int
findrslot()
{
   register int    i;

   /* look for tester slot first */
   i = findtestslot();
   if (i > -1)
      return i;

   for (i = 0; i < MAXPLAYER; i++) {
      if (players[i].p_status == PFREE) {	/* We have a free slot */
	 players[i].p_status = POUTFIT;	/* possible race condition */
	 break;
      }
   }
   if ((i == MAXPLAYER) || (i == -1)) {
      if (debug) {
	 fprintf(stderr, "No more room in game\n");
      }
      return -1;
   }
   bzero(&players[i].p_stats, sizeof(struct stats));
   players[i].p_stats.st_tticks = 1;
   return (i);
}

void
warning(f, mess)
   int		f;
   char 	*mess;
{
   if (debug)
      fprintf(stderr, "warning: %s\n", mess);
}

void
shipconfig()
{
   myship->s_turns = 310000;
   myship->s_accint = 200;
   myship->s_decint = 200;
   myship->s_warpcost = 0;
   myship->s_accs = 100;
   myship->s_repair = 30000;
   myship->s_maxfuel = 60000;
   myship->s_maxshield = 30000;
   myship->s_maxdamage = 30000;
   myship->s_mass = 5000;
   myship->s_phasercost = 0;
   myship->s_torpcost = 0;
   myship->s_cloakcost = 0;
   myship->s_torpfuse = 1000000;
   myship->s_torpdamage = 0;
   myship->s_plasmadamage = 0;
   myship->s_plasmacost = 0;
   myship->s_plasmaturns = 0;
   myship->s_plasmaspeed = 10;
   myship->s_plasmafuse = 0;
   myship->s_phaserfuse = 0;
   myship->s_wpncoolrate = 1000;
   myship->s_egncoolrate = 1000;
   myship->s_maxspeed = 20;
   myship->s_recharge = 1000;
   me->p_shield = me->p_ship.s_maxshield;
}

int
killrobots()
{
   register        i;
   register struct player *j;
   int             r, it = 0;

   do {
      it++;
      r = 0;

      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if (j->p_status == PFREE)
	    continue;
#ifdef BIRD
	 if (j->p_flags & PFBIRD) {
	    if (debug)
	       printf("killing bird\n");
#ifdef DS
	    if(j->p_process > 1)
	       (void)kill(j->p_process, SIGTERM);
#else
	    j->p_status = PFREE;
#endif
	    r++;
	    continue;
	 }
#endif
#ifdef SNAKEPATROL
	 if (j->p_flags & PFSNAKE) {
	    if (debug)
	       printf("killing snake\n");
#ifdef DS
	    if(j->p_process > 1)
	       (void)kill(j->p_process, SIGTERM);
#else
	    j->p_status = PFREE;
#endif
	    r++;
	    continue;
	 }
#endif
	 if (j->p_flags & PFROBOT) {
	    if (j->p_status != PEXPLODE) {
	       if (debug)
		  printf("killing robot\n");
	       j->p_status = PEXPLODE;
	       j->p_explode = 10;
	    }
	    r++;
	 }
      }
      if(r) 
	 sleep(5);
      
   } while (r && it < 10);
   return r == 0;
}

void
borgenter(tno, pno, s_type)
   int             tno;
   int             pno;
   int             s_type;
{
   static int      lastteam = -1;
   char            buf[80];
   char            addrbuf[10];

   getship(myship, s_type);
   me->p_updates = 0;
   me->p_flags = PFDOCKOK;
   me->p_speed = me->p_desspeed = 0;
   me->p_team = 0;
   me->p_ntorp = 0;
   me->p_cloakphase = 0;
   me->p_nplasmatorp = 0;
   me->p_damage = 0;
   me->p_subdamage = 0;
   me->p_etemp = 0;
   me->p_etime = 0;
   me->p_wtemp = 0;
   me->p_wtime = 0;
   me->p_subshield = 0;
   me->p_swar = 0;
   me->p_hostile &= ~me->p_team;
   me->p_kills = 0.0;
   me->p_armies = 0;
#ifdef SB_DOCK
   me->p_numdocked = 0;
#else
   me->p_docked = 0;
#endif
   me->p_port[0] = VACANT;
   me->p_port[1] = VACANT;
   me->p_port[2] = VACANT;
   me->p_port[3] = VACANT;

   me->p_fuel = myship->s_maxfuel;

   me->p_x = GWIDTH/2;
   me->p_y = GWIDTH/2;

   me->p_dir = me->p_desdir = 0;

   sprintf(buf, "%c%c", teamlet[me->p_team], shipnos[me->p_no]);
   strncpy(me->p_mapchars, buf, 2);

   if (lastteam != tno) {
      lastteam = tno;
      sprintf(buf, "%.16s joining as %2.2s (%s)",
	      me->p_name,
	      me->p_mapchars,
	      me->p_login);
      sprintf(addrbuf, " %c%c->ALL",
	      teamlet[me->p_team], shipnos[me->p_no]);
      pmessage2(buf, 0, MALL, addrbuf, me->p_no);
   }
}

int
plname_to_index(name)

   char	*name;
{
   register			i, len = strlen(name);
   register struct planet	*l;

   for(i=0,l=planets; i< MAXPLANETS; i++,l++){
      if(strncasecmp(l->pl_name, name, len)==0)
	 return i;
   }
   return -1;
}
