/*
 * clockmove.c
 */
#include "copyright.h"

#include <stdio.h>
#include <signal.h>
#include <values.h>
#include <math.h>
#include <time.h>
#include "defs.h"
#include "struct.h"
#include "data.h"

/* how often to check for players in the game in cyles (1/10th second) */
#define PLAYERCHECK	100

extern struct player *perfs[2];
extern int      num_perfs;
extern int      debug;
extern int      lastm;
extern int      tno1, tno2;
extern int      team1, team2;
extern int      patrol;
extern int      xpos, ypos;
extern int      clockspan;

static int      c_clock;
static          _move();
static int      defenders[ALLTEAM];

unsigned char   getacourse();
unsigned char   timedir();

clockmove()
{
   register int    i, moved = 0;

   c_clock++;

   _move();
}

static
_move()
{
   if (!perfs[0] || !perfs[1])
      exitClock();

   /* keep ghostbuster away */
   perfs[0]->p_ghostbuster = 0;
   perfs[1]->p_ghostbuster = 0;

#ifdef nodef
   perfs[0]->p_status = PDEAD;
   perfs[1]->p_status = PDEAD;
   perfs[0]->p_explode = 100;
   perfs[1]->p_explode = 100;
#endif

   if (c_clock == 5)
      startclock();
   else if (c_clock > 5)
      moveclock();

   return 1;
}

static struct phaser *hour, *min;
static struct plasmatorp *second, *twelve_mark;
static struct torp *hour_marks[12];

startclock()
{
   struct player  *p1 = perfs[0], *p2 = perfs[1];
   unsigned char   hour_dir, min_dir;
   register        c, i, lc;
   register struct torp *t;

   p1->p_x = p2->p_x = xpos;
   p1->p_y = p2->p_y = ypos;

   for (c = 0, i = p1->p_no * MAXTORP, t = &torps[i]; i < p1->p_no * MAXTORP + MAXTORP; c++, i++, t++) {
      hour_marks[c] = t;
      t->t_no = i;
      t->t_owner = p1->p_no;
      t->t_team = ROM;
      t->t_dir = 0;
      t->t_damage = 0;
      t->t_speed = 0;
      t->t_fuse = MAXINT;
      t->t_turns = 0;
      t->t_team = p1->p_team;
   }
   p1->p_ntorp = 8;

   for (c = 8, i = p2->p_no * MAXTORP, t = &torps[i]; c < 12; c++, i++, t++) {
      hour_marks[c] = t;
      t->t_no = i;
      t->t_owner = p2->p_no;
      t->t_team = ROM;
      t->t_dir = 0;
      t->t_damage = 0;
      t->t_speed = 0;
      t->t_fuse = MAXINT;
      t->t_turns = 0;
      t->t_team = p2->p_team;
   }
   p1->p_ntorp = 4;

   hour = &phasers[p1->p_no];

   min = &phasers[p2->p_no];

   second = &plasmatorps[p1->p_no * MAXPLASMA];

   second->pt_no = p1->p_no * MAXPLASMA;
   second->pt_status = PTMOVE;
   second->pt_owner = p1->p_no;
   second->pt_team = p1->p_team;
   second->pt_damage = 0;
   second->pt_speed = 0;
   second->pt_war = 0;
   second->pt_fuse = 20;
   second->pt_turns = 0;

   twelve_mark = &plasmatorps[p2->p_no * MAXPLASMA];

   twelve_mark->pt_no = p1->p_no * MAXPLASMA;
   twelve_mark->pt_status = PTMOVE;
   twelve_mark->pt_owner = p2->p_no;
   twelve_mark->pt_team = p2->p_team;
   twelve_mark->pt_damage = 0;
   twelve_mark->pt_speed = 0;
   twelve_mark->pt_war = 0;
   twelve_mark->pt_fuse = 20;
   twelve_mark->pt_turns = 0;
   twelve_mark->pt_x = xpos + (clockspan + 300) * Cos[0];
   twelve_mark->pt_y = ypos + (clockspan + 300) * Sin[0];

   p2->p_nplasmatorp = 1;

   /* position the hour marks */

   lc = 0;
   for (i = 0; i < 12; i++) {
      if ((i % 3) == 0)
	 c = 8 + i / 3;
      else
	 c = lc++;

      hour_marks[c]->t_status = TMOVE;
      hour_marks[c]->t_x = xpos + clockspan * Cos[timedir(i, 12)];
      hour_marks[c]->t_y = ypos + clockspan * Sin[timedir(i, 12)];
   }

   if (debug)
      fprintf(stderr, "started\n");
}

moveclock()
{
   register        i, x, y;
   register struct player *j;
   struct player  *p1 = perfs[0], *p2 = perfs[1];
   int             ok;
   unsigned char   hour_dir, min_dir, sec_dir;

   if ((c_clock % PLAYERCHECK) == 0) {
      /*
       * every x seconds make sure there's people in the game
       */
      ok = 0;
      defenders[FED] = defenders[ROM] = defenders[KLI] = defenders[ORI] = 0;
      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if ((j->p_status != PFREE) && !(j->p_flags & PFROBOT)) {
	    ok = 1;
	    defenders[j->p_team]++;
	 }
      }
      if (!ok)
	 exitClock();
   }
   /* other checks here get performed every update instead of every second */

   if (p1->p_damage > 0)
      p1->p_damage = 0;
   if (p2->p_damage > 0)
      p2->p_damage = 0;

   if ((c_clock % 10) != 0)
      return;

   for (i = 0; i < 12; i++) {
      hour_marks[i]->t_status = TMOVE;
      hour_marks[i]->t_fuse = 20;
   }

   get_time(&hour_dir, &min_dir, &sec_dir);

   x = xpos + ((2 * (clockspan - 300)) / 3) * Cos[hour_dir];
   y = ypos + ((2 * (clockspan - 300)) / 3) * Sin[hour_dir];

   if (x != hour->ph_x || y != hour->ph_y)
      hour->ph_target++;

   hour->ph_x = x;
   hour->ph_y = y;
   hour->ph_fuse = 20;
   hour->ph_status = PHHIT2;
   hour->ph_dir = hour_dir;
   p1->p_dir = p1->p_desdir = hour_dir;

   x = xpos + (clockspan - 300) * Cos[min_dir];
   y = ypos + (clockspan - 300) * Sin[min_dir];

   if (x != min->ph_x || y != min->ph_y)
      min->ph_target++;

   min->ph_x = x;
   min->ph_y = y;
   min->ph_fuse = 20;
   min->ph_status = PHHIT2;
   min->ph_dir = min_dir;
   min->ph_target++;
   p2->p_dir = p2->p_desdir = min_dir;

   second->pt_status = PTMOVE;
   second->pt_fuse = 20;
   second->pt_x = xpos + (clockspan - 150) * Cos[sec_dir];
   second->pt_y = ypos + (clockspan - 150) * Sin[sec_dir];

   twelve_mark->pt_status = PTMOVE;
   twelve_mark->pt_fuse = 20;
}

get_time(hdir, mdir, sdir)
   unsigned char  *hdir, *mdir, *sdir;
{
   time_t          now = time(NULL);
   struct tm      *tmstruct;
   int             hour, minute, second;

   tmstruct = localtime(&now);

   hour = tmstruct->tm_hour;
   if (hour > 12)
      hour -= 12;
   minute = tmstruct->tm_min;
   second = tmstruct->tm_sec;

   *hdir = timedir(hour, 12);
   *mdir = timedir(minute, 12 * 5);
   *sdir = timedir(second, 12 * 5);
}

unsigned char
timedir(i, r)
   int             i, r;
{
   /* return direction on clock for value between r */
   int             v = (i * 256) / r;
   return (unsigned char) v;
}

exitClock(sig)
   int             sig;
{
   register int    i;
   register struct player *j;
   register struct torp *k;
   register struct plasmatorp *f;

   signal(SIGALRM, SIG_DFL);

   if (debug)
      fprintf(stderr, "clock exiting\n");

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {
	 /* torps are free here */
	 if (k->t_status != TFREE)
	    k->t_status = TOFF;
      }
      f = &plasmatorps[j->p_no * MAXPLASMA];
      f->pt_status = PTFREE;
      /* plasma */
   }
   bzero(&phasers[perfs[0]->p_no], sizeof(struct phaser));
   bzero(&phasers[perfs[1]->p_no], sizeof(struct phaser));

   bzero(perfs[0], sizeof(struct player));
   bzero(perfs[1], sizeof(struct player));
   perfs[0]->p_stats.st_tticks = 1;
   perfs[1]->p_stats.st_tticks = 1;

   /*
    * note: if we had a segmentation violation or bus error we want a core
    * file to debug
    */
   if (sig == SIGBUS || sig == SIGSEGV)
      abort();
   else
      exit(0);
}

pmessage(str, recip, group, address)
   char           *str;
   int             recip;
   int             group;
   char           *address;
{
   pmessage2(str, recip, group, address, 255);
}

pmessage2(str, recip, group, address, from)
   char           *str;
   int             recip;
   int             group;
   char           *address;
   unsigned char   from;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = from;
   (void) sprintf(cur->m_data, "%-9s %s", address, str);
   cur->m_flags |= MVALID;
}

/* get course from (mx,my) to (x,y) */
unsigned char 
getacourse(x, y, mx, my)
   int             x, y, mx, my;
{
   unsigned char	d;
   if (x == mx && y == my)
      return 0;

   d = (unsigned char) (atan2((double) (x - mx),
				 (double) (my - y)) / 3.14159 * 128.);
   MATH_ERR_CHECK(x-mx+my-y);
   return d;
}

/* return rnd between -range & range */
rrnd(range)
   int             range;
{
   return random() % (2 * range) - range;
}

/*
 * gives approximate distance from (x1,y1) to (x2,y2) with only
 * overestimations, and then never by more than (9/8) + one bit uncertainty.
 */

int 
ihypot(x1, y1)
   int             x1, y1;
{
   register        x2 = 0, y2 = 0;

   if ((x2 -= x1) < 0)
      x2 = -x2;
   if ((y2 -= y1) < 0)
      y2 = -y2;
   return (x2 + y2 - (((x2 > y2) ? y2 : x2) >> 1));
}
