
/* taken mostly from update.c, guzzler distribution,
 * Nick Trown (trown@ecst.csuchico.edu)
 */

#include <stdio.h>
#ifdef SYSV
#include <string.h>
#include <sys/fcntl.h>		/* 18/8/92 SYSV include SK */
#include <time.h>
#else
#include <strings.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/wait.h>
#include <sys/resource.h>
#include <signal.h>
#include <sys/ioctl.h>
#ifdef hpux
#include <sys/ptyio.h>
#endif
#include <errno.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>

#define RSA_KEYCAP		"./rsa_keycap"
#define RSA_KEYCOMP		"./rsa_keycomp"
#define MOTD_KEYS		"./motd/rsa_motd"
/*
#define METASERVER		"129.212.11.1"
*/
#define METASERVER		"metaserver.ecst.csuchico.edu"

#define METAPORT		3530
#define MAXTRIES		5

#define SLEEPTIME		5

#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* rsa_getkeys.c */
static int open_port P_((char *host, int port, int verbose));
static void backup P_((char *f));

#ifdef USE_SYS_PROTOS

extern int                    printf P_((const char *, ...));
extern long unsigned int      inet_addr P_((/* ??? */));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void                   perror P_((const char *));
extern size_t                 fwrite P_((const void *, size_t, size_t, FILE *));
extern int                    fflush P_((FILE *));
extern int                    fclose P_((FILE *));
extern void *                 malloc P_((size_t));
extern void                   free P_((void *));
extern int                    connect P_((int s, struct sockaddr * name, int namelen));
extern int                    socket P_((int domain, int type, int protocol));

#endif

#undef P_


/*
    The connection to the metaserver is by Andy McFadden.
    This calls the metaserver and parses the output into something
    useful
*/

static int
open_port(host, port, verbose)
   char                           *host;
   int                             port;
   int                             verbose;
{
   struct sockaddr_in              addr;
   struct hostent                 *hp;
   int                             sock;
   /* Connect to the metaserver */
   /* get numeric form */
   if ((addr.sin_addr.s_addr = inet_addr(host)) == -1) {
      if ((hp = gethostbyname(host)) == NULL) {
	 if (verbose)
	    fprintf(stderr, "unknown host '%s'\n", host);
	 return (-1);
      } else {
	 addr.sin_addr.s_addr = *(long *) hp->h_addr;
      }
   }
   addr.sin_family = AF_INET;
   addr.sin_port = htons(port);
#ifdef DEBUG
   printf("Opening socket at (%s,%d)\n", host, port);
#endif
   if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      if (verbose)
	 perror("rsa_getkeys'socket");
      return (-1);
   }
   if (connect(sock, (struct sockaddr *) & addr, sizeof(addr)) < 0) {
      if (verbose)
	 perror("rsa_getkeys'connect");
      close(sock);
      return (-1);
   }
   return (sock);
}

void
main(argc, argv)
   int                             argc;
   char                          **argv;
{
   int                             tries;
   FILE                           *keyfile;
   int                             sock;
   int                             num_read, num_written;
   char				   buf[BUFSIZ];

   for (tries = 0; (tries < MAXTRIES); tries++) {

#ifdef DEBUG
      printf("Try number %d to connect to (%s,%d)\n", tries, METASERVER, METAPORT);
#endif

      if ((sock = open_port(METASERVER, METAPORT, 1)) > 0) {

	 backup(RSA_KEYCAP);
	 if ((keyfile = fopen(RSA_KEYCAP, "w")) == NULL){
	 perror(RSA_KEYCAP);
	    exit(1);
	 }
	 while ((num_read = read(sock, buf, BUFSIZ)) != 0) {
	    num_written = 0;
	    do {
	       num_written += fwrite(buf + num_written,
				     sizeof(*buf),
		   num_read - num_written, keyfile);
#ifdef DEBUG
	       printf("Wrote %d bytes\n", num_written);
#endif
	    } while (num_written < num_read);
	 }
	 fflush(keyfile);
	 fclose(keyfile);
	 close(sock);

	 sprintf(buf, "%s -m %s %s", 
	      RSA_KEYCOMP, MOTD_KEYS, RSA_KEYCAP);
#ifdef DEBUG
	 printf("Exec'ing %s\n", buf);
#endif
	 execl("/bin/sh", "sh", "-c", buf, 0);
	 _exit(0);	/* shouldn't be reached */
      }
      sleep(SLEEPTIME);
   }
   exit(0);
}

static void 
backup(f)

   char *f;
{
   char         *backup_f;
   struct stat  sbuf;

#ifdef DEBUG
   printf("Backing up %s\n",f);
#endif

   if(stat(f, &sbuf) < 0)
      return;
   backup_f = (char *)malloc(strlen(f)+2);
   if(!backup_f) { perror("rsa_getkeys'malloc"); exit(1); }
   sprintf(backup_f, "%s~", f);
   (void)link(f, backup_f);
   (void)unlink(f);
   free((char *)backup_f);
}
